@@ -27,9 +27,9 @@
 #include <linux/random.h>
 #include <linux/moduleloader.h>
 #include <linux/bpf.h>
+#include <linux/nospec.h>
 
 #include <asm/unaligned.h>
-#include <asm/barrier.h>
 
 /* Registers */
 #define BPF_R0	regs[BPF_REG_0]
@@ -632,7 +632,7 @@ select_insn:
 		DST = IMM;
 		CONT;
 	LD_IMM_DW:
-		osb();
+		barrier_nospec();
 		DST = (u64) (u32) insn[0].imm | ((u64) (u32) insn[1].imm) << 32;
 		insn++;
 		CONT;
@@ -847,7 +847,7 @@ out:
 		*(SIZE *)(unsigned long) (DST + insn->off) = IMM;	\
 		CONT;							\
 	LDX_MEM_##SIZEOP:						\
-		osb();							\
+		barrier_nospec();					\
 		DST = *(SIZE *)(unsigned long) (SRC + insn->off);	\
 		CONT;
 
